function [ fingertips ] = getFingertips( x_cont, y_cont, comX, comY, display_info )
%getFingertips Get the coordinates of the fingertips.
%   x_cont - points along contour of hand/arm (x values)
%   y_cont - points along contour of hand/arm (y values)
%   comX - center of mass (x-value)
%   comY - center of mass (y-value)
%   display_info - display infor about computations made in this function

xpts = x_cont';
ypts = y_cont';

[unused num_points] = size(xpts);

%get distance to each point
distances = [];

skipVal = 5;

for i=1:skipVal:num_points,
    distance = (xpts(1,i)-comX)*(xpts(1,i)-comX) + (ypts(1,i) - comY)*(ypts(1,i) - comY);
    distance = sqrt(distance);
    distances = [distances distance];
end

%shift data so first element is minimum
[unused ind] = min(distances);
distances = circshift(distances,[0 -(ind + 1)]);

%smooth out values
distances = smooth(distances);

disp('     Computing potential fingertip locations...');
[peak_values peak_index] = findpeaks(distances, 'MINPEAKDISTANCE', 20, 'SORTSTR', 'descend');
[web_values web_index] = findpeaks(-1*(distances-max(distances)), 'MINPEAKDISTANCE', 20, 'SORTSTR', 'descend');

web_values = max(distances) - web_values;

[num_web unused] = size(web_values);
num_web = min(3, num_web);
web_avg = mean(web_values(1:num_web,1));
web_min = min(web_values(1:num_web,1));

[num_distances unused] = size(distances);

peak_index = mod((peak_index + ind + 1), num_distances);

fingertips = [];

[num_peaks unused] = size(peak_index);

disp('     Filtering potential fingertip locations...');
%reduce potential fingertip matches
peak_coords = [];
peak_avg = 0;
peak_count = 0;
max_peak = 0;
for i=1:num_peaks,
    xVal = xpts(1, (peak_index(i,1)*skipVal + 1));
    if(xVal < comX)
        yVal = ypts(1, (peak_index(i,1)*skipVal + 1));
        peak_coords = [peak_coords  [xVal yVal]'];
        temp_dist = sqrt((xVal-comX)*(xVal-comX) + (yVal - comY)*(yVal - comY));
        if(temp_dist > max_peak)
           max_peak = temp_dist; 
        end
        peak_avg = peak_avg + temp_dist;
        peak_count = peak_count + 1;
    end
end

peak_avg = peak_avg/peak_count;
tip_web_avg = mean([max_peak web_min]);

%reduce potential fingertip matches
for i=1:peak_count,
    xVal = peak_coords(1,i);
    yVal = peak_coords(2,i);
    tip_dist = (xVal-comX)*(xVal-comX) + (yVal - comY)*(yVal - comY);
    tip_dist = sqrt(tip_dist);
    if(tip_dist < 1.1*tip_web_avg || tip_dist < 1.5*web_avg)
        continue
    end
    fingertips = [fingertips [xVal yVal]'];
end

if(display_info)
    figure(length(findobj('Type','figure')) + 1)
    hold on
    title('Distance vs Ordered Contour Point');
    plot(distances);
    hold off
    
    figure(length(findobj('Type','figure')) + 1)
    hold on
    scatter(xpts,ypts, 1);
    title('Contour Plot with Fingertip Locations');
    plot(comX, comY, 'o');
    [unused num_tips] = size(fingertips);
    if(num_tips > 0)
        plot(fingertips(1,:), fingertips(2,:), 'ks');
    end
    camroll(270)
    hold off
end

end

