function [ hand_image, x_cont, y_cont, comX, comY ] = isolateHand( post_seg_image, display_info )
%isolateHand Isolate the hand from skin segmented binary image 
%   post_seg_image - binary image, post segmentation
%   display_info - display graphs concerning computations made in this
%   function

disp('     Softening edges...');
%blur image to soften edges
f = fspecial('gaussian', 31, 8);
post_seg_image = imfilter(post_seg_image, f, 'conv');

%get size of image
[x y z] = size(post_seg_image);

%set pixels to black if below some threshold
for i=1:x,
    for j=1:y,
       if(post_seg_image(i,j) < 60)
           post_seg_image(i,j) = 0;
       else
           post_seg_image(i,j) = 255;
       end
    end 
end

%get the boundaries of the blobs in the image (ignore holes in blobs)
B = bwboundaries(post_seg_image, 8, 'noholes');

%set up variables
[blobs unused] = size(B);
maxSize = 0;
currentSize = 0;
handBoundary = [];
potentialMatch = [];

disp('     Finding hand...');
%get largest blob (corresponds to hand)
for blob = 1:blobs,
   potentialMatch = B{blob,1};
   [currentSize unused] = size(potentialMatch);
   if(currentSize > maxSize)
      handBoundary = potentialMatch; 
      maxSize = currentSize;
   end
end

%place points in clockwise order
[xpts ypts] = poly2cw(handBoundary(:,1), handBoundary(:,2));

[unused num_pts] = size(xpts);
skipVal = 5;

%set pixels on interior of hand contour to white
[x y z] = size(post_seg_image);

mask = poly2mask(xpts, ypts, x, y);
for i=1:x,
    for j=1:y,
        if(mask(i,j) == 1)
            post_seg_image(j,i) = 255;
        else
            post_seg_image(j,i) = 0;
        end
    end 
end

post_seg_image = post_seg_image(:,1:y);

%set up variables
horzdist = 0;
isWrist = false;
widths = [];
width_count = 0;
skipVal = 5;

disp('     Locating wrist...');
%get width of forearm/wrist/hand
for i=0:skipVal:(x-1),
    for j=1:skipVal:y,
        if(post_seg_image(x-i,j) > 0 && ~isWrist)
            horzdist = j;
            isWrist = true;
        end
        if(post_seg_image(x-i,j) == 0 && isWrist)
            horzdist = -1*(horzdist - j);
            break
        end
    end
    if(isWrist)
       widths = [widths  horzdist];
       width_count = width_count + 1;
       isWrist = false;
    end
end

%locate position of wrist
widths = smooth(smooth(widths));
max_width = max(widths);
widths_inverted = -1*(widths - max_width);
[unused wristIndexLocation] = findpeaks(widths_inverted(1:floor(55*width_count/100), 1), 'MINPEAKDISTANCE', 5, 'SORTSTR', 'descend');

[unused foundWrist] = size(wristIndexLocation);
if(foundWrist)
    %remove wrist
    post_seg_image((x-skipVal*wristIndexLocation(1,1)):x,:) = 0;

    B = bwboundaries(post_seg_image);
    handBoundary = B{1,1};
    [xpts ypts] = poly2cw(handBoundary(:,1), handBoundary(:,2));

    if(display_info)
        figure(length(findobj('Type','figure')) + 1)
        hold on
        title('Wrist Width vs Ordered Contour Point');
        plot(widths)
        scatter(wristIndexLocation(1,1), widths(wristIndexLocation(1,1),1),'o')
        hold off
    end
end

%set up variables
comX = 0;
comY = 0;
count = 0;

%set pixels to black if below some threshold
for i=1:x,
    for j=1:y,
       if(post_seg_image(i,j) > 0)
           comX = comX + i;
           comY = comY + j;
           count = count + 1;
       end
    end 
end

%get center of mass of hand
comX = comX/count;
comY = comY/count;

%set return variables
hand_image = post_seg_image;
x_cont = xpts;
y_cont = ypts;

end

