function [ imout ] = segmentSkinHSV( im, trainData )
%segmentSkinHSV Segment skin
%   *im - image
%   *trainData - learned data in form of [h s v] where h, s, and v are
%   column vectors

im = imresize(im,0.5);

[x y z] = size(im);

imout = rgb2gray(im);
im = rgb2hsv(im);

totalCountH = sum(trainData(:,1));
totalCountS = sum(trainData(:,2));
totalCountV = sum(trainData(:,3));

disp('     Blurring image...');
f = fspecial('gaussian', 31, 5);
im = imfilter(im, f, 'conv');

[numBins unused] = size(trainData);

binSize = 1/numBins;

disp('     Segmenting skin from image...');
for i=1:x,
    for j=1:y,
        valH = im(i,j,1);
        valS = im(i,j,2);
        valV = im(i,j,3);
                
        binH = ceil(valH/binSize);
        binS = ceil(valS/binSize);
        binV = ceil(valV/binSize);
        
        if(binH == 0) binH = binH + 1; end
        if(binS == 0) binS = binS + 1; end
        if(binV == 0) binV = binV + 1; end
        
        skinValCountH = trainData(binH, 1);
        skinValCountS = trainData(binS, 2);
        skinValCountV = trainData(binV, 3);
        
        probH = skinValCountH/totalCountH;
        probS = skinValCountS/totalCountS;
        probV = skinValCountV/totalCountV;
        
        if(probH > 0.001 && probS > 0.001 && probV > 0.001)
            imout(i,j,1) = 255;
        else
            imout(i,j,1) = 0;
        end
    end
end

end